/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or (at
 *  your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *  MA 02110-1301, USA.
 *
 *  Copyright © 2006 Pierre Habouzit
 */

#ifndef MUTT_LIB_LIB_STR_H
#define MUTT_LIB_LIB_STR_H

#include <string.h>
#include <ctype.h>

#include "../lib/str.h"

#include "mem.h"

#define HUGE_STRING     5120
#define LONG_STRING     1024
#define STRING          256
#define SHORT_STRING    128

#define NONULL(x) (x?x:"")
#define ISSPACE(c) isspace((unsigned char)c)

extern unsigned char const __m_strdigits[128];
extern signed char const __m_b64digits[128];
extern char const __m_b64chars[64];

extern char const __m_b36chars_lower[36];
extern char const __m_b36chars_upper[36];

/****************************************************************************/
/* char related                                                             */
/****************************************************************************/

static inline int hexval(int c) {
    return !(c & ~127) && __m_strdigits[c] < 16 ? __m_strdigits[c] : -1;
}

static inline int base64val(int c) {
    return (c & ~127) ? -1 : __m_b64digits[c];
}

/****************************************************************************/
/* length related                                                           */
/****************************************************************************/

static inline int m_strisempty(const char *s) {
    return !s || !*s;
}

static inline ssize_t m_strlen(const char *s) {
    return s ? strlen(s) : 0;
}

static inline ssize_t m_strnlen(const char *s, ssize_t n) {
    const char *p = memchr(s, '\0', n);
    return p ? p - s : n;
}

/****************************************************************************/
/* comparisons                                                              */
/****************************************************************************/

static inline int m_strcmp(const char *a, const char *b) {
    return strcmp(NONULL(a), NONULL(b));
}

static inline int m_strcasecmp(const char *a, const char *b) {
    return strcasecmp(NONULL(a), NONULL(b));
}

static inline int m_strncmp(const char *a, const char *b, size_t n) {
    return strncmp (NONULL(a), NONULL(b), n);
}

static inline int m_strncasecmp(const char *a, const char *b, size_t n) {
    return strncasecmp(NONULL(a), NONULL(b), n);
}

/****************************************************************************/
/* making copies                                                            */
/****************************************************************************/

static inline char *m_strdup(const char *s) {
    ssize_t len = m_strlen(s);
    return len ? p_dup(s, len + 1) : NULL;
}

static inline char *m_strreplace(char **p, const char *s) {
    p_delete(p);
    return (*p = m_strdup(s));
}

ssize_t m_strcpy(char *dst, ssize_t n, const char *src);
ssize_t m_strncpy(char *dst, ssize_t n, const char *src, ssize_t l);

static inline ssize_t m_strcat(char *dst, ssize_t n, const char *src) {
    ssize_t dlen = m_strnlen(dst, n - 1);
    return dlen + m_strcpy(dst + dlen, n - dlen, src);
}

static inline ssize_t
m_strncat(char *dst, ssize_t n, const char *src, ssize_t l) {
    ssize_t dlen = m_strnlen(dst, n - 1);
    return dlen + m_strncpy(dst + dlen, n - dlen, src, l);
}

/****************************************************************************/
/* parsing related                                                          */
/****************************************************************************/

static inline const char *skipspaces(const char *s) {
    while (*s && isspace((unsigned char)*s))
        s++;
    return s;
}
static inline char *vskipspaces(const char *s) {
    return (char *)skipspaces(s);
}

#endif /* MUTT_LIB_LIB_STR_H */
